package com.chelvc.cloud.vehicle.api.param;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
 * 商家认证数据模型
 *
 * @author liude
 * @data 2023/12/10
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class MerchantAuthModifyParam implements Serializable {
    /**
     * 主键ID
     */
    private Long id;
    /**
     * 营业执照编号
     */
    @NotEmpty(message = "营业执照编号不能为空")
    @Size(max = 100, message = "营业执照编号长度不能大于64")
    private String businessLicenseId;
    /**
     * 营业执照证件号
     */
//    @NotEmpty(message = "营业执照证件号不能为空")
    @Size(max = 100, message = "营业执照证件号长度不能大于64")
    private String businessLicenseNumber;
    /**
     * 营业执照有效期
     */
    @NotNull(message = "营业执照有效期不能为空")
    private Date businessLicenseExpirationDate;
    /**
     * 身份证号
     */
    @NotEmpty(message = "身份证号不能为空")
    @Size(max = 100, message = "身份证号长度不能大于32")
    private String idCardNumber;
    /**
     * 身份证有效期
     */
    @NotNull(message = "身份证有效期不能为空")
    private Date idCardExpirationDate;
    /**
     * 店铺图
     */
    private String storeImage;
    /**
     * 法人姓名
     */
    @NotEmpty(message = "法人姓名不能为空")
    @Size(max = 100, message = "法人姓名长度t不能大于255")
    private String legalRepresentativeName;
    /**
     * 法人性别
     */
//    @NotEmpty(message = "法人性别不能为空")
    @Size(max = 100, message = "营业执照编号长度不能大于1")
    private String legalRepresenativeGender;
    /**
     * 手机号
     */
    @NotEmpty(message = "手机号不能为空")
    @Size(max = 100, message = "手机号长度不能大于20")
    private String mobileNumber;
    /**
     * 账号
     */
    @Size(max = 100, message = "账号长度不能大于64")
    private String account;
    /**
     * 账号注册日期
     */
    private Date accountRegistrationDate;
    /**
     * 店铺类型
     */
    private String storeType;
    /**
     *店名
     */
    @NotEmpty(message = "店名不能为空")
    @Size(max = 100, message = "店名长度不能大于255")
    private String storeName;
    /**
     * 店铺地址
     */
    @Size(max = 100, message = "店铺地址长度不能大于255")
    private String storeAddress;
    /**
     * 经营许可证照片
     */
    private String businessLicensePhoto;
    /**
     * 经营许可证号
     */
    @NotEmpty(message = "经营许可证号不能为空")
    @Size(max = 100, message = "经营许可证号长度不能大于20")
    private String businessLicenseNumberValid;
    /**
     * 经营许可证有效期
     */
    @NotNull(message = "经营许可证有效期不能为空")
    private Date businessLicenseExpirationDateValid;
    /**
     * 推荐人
     */
    private String recommendationUser;

    /**
     * 营业执照证件号
     */
    private String businessPhoto;
    /**
     * 身份证正面照片
     */
    private String idCardFrontPhoto;

    /**
     * 身份证反面照片
     */
    private String idCardBackPhoto;

    /**
     * 店铺内部环境图
     */
    private String storeInnerImage;


    /**
     * 店铺详细地址
     */
    @Size(max = 255, message = "店铺详细地址长度不能大于255")
    private String storeAddressDetail;

    /**
     * 邮箱
     */
    @Size(max = 255, message = "邮箱长度不能大于255")
    private String email;
}
